package com.caimei.controller;

import com.caimei.model.ResponseJson;
import com.caimei.model.dto.CouponsDto;
import com.caimei.model.vo.CouponVo;
import com.caimei.model.vo.ProductVo;
import com.caimei.service.CouponService;
import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Aslee
 * @date : 2021/10/26
 */
@Api(tags = "优惠券API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/coupon")
public class CouponApi {
    private final CouponService couponService;


    @ApiOperation("首页优惠券弹窗展示")
    @ApiImplicitParam(name = "userId", value = "用户id", required = false)
    @GetMapping("/display")
    public ResponseJson<Map<String,Integer>> couponDisplay(Integer userId) {
        return couponService.couponDisplay(userId);
    }

    @ApiOperation("领券中心")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "userId", value = "机构用户id"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/center")
    public ResponseJson<PageInfo<CouponVo>> couponCenter(Integer userId, @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                               @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        return couponService.couponCenter(userId, pageNum, pageSize);
    }
    @ApiOperation("领取优惠券")
    @PostMapping("/collar")
    public ResponseJson<String> collarCoupons(@RequestBody CouponsDto couponsDto) {
        if (couponsDto.getCouponId() == null || couponsDto.getUserId() == null) {
            return ResponseJson.error("参数异常", null);
        }
        return couponService.collarCoupons(couponsDto);
    }

    @ApiOperation("已领取优惠券中心")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "userId", value = "机构用户id"),
            @ApiImplicitParam(required = false, name = "status", value = "使用状态: 1未使用 2已使用 3已失效"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/receive/list")
    public ResponseJson<PageInfo<CouponVo>> receiveCouponList(Integer userId, Integer status,
                                                         @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                         @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (userId == null || status == null) {
            return ResponseJson.error("参数异常", null);
        }
        return couponService.receiveCouponList(userId, status, pageNum, pageSize);
    }

    @ApiOperation("商品详情优惠券")
    @ApiImplicitParams({
            @ApiImplicitParam(required = false, name = "productId", value = "机构用户id"),
            @ApiImplicitParam(required = false, name = "userId", value = "机构用户id")
    })
    @GetMapping("/product/detail")
    public ResponseJson<List<CouponVo>> productDetailCoupons(Integer productId, Integer userId) {
        if (productId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return couponService.productDetailCoupons(productId, userId);
    }

    @ApiOperation("统计已领取优惠券数量")
    @ApiImplicitParam(required = true, name = "userId", value = "机构用户id")
    @GetMapping("/receive/count")
    public ResponseJson<Map<String, Integer>> receiveCount(Integer userId) {
        if (userId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return couponService.receiveCount(userId);
    }

    /*@ApiOperation("优惠券部分商品数据")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "couponId", value = "优惠券id"),
            @ApiImplicitParam(required = true, name = "userId", value = "用户id"),
            @ApiImplicitParam(required = false, name = "productName", value = "商品名称"),
            @ApiImplicitParam(required = false, name = "pageNum", value = "页码"),
            @ApiImplicitParam(required = false, name = "pageSize", value = "每页数量")
    })
    @GetMapping("/product/page")
    public ResponseJson<PageInfo<ProductVo>> productPage(Integer couponId, Integer userId, String productName,
                                                         @RequestParam(value = "pageNum", defaultValue = "1") int pageNum,
                                                         @RequestParam(value = "pageSize", defaultValue = "10") int pageSize) {
        if (couponId == null || userId == null) {
            return ResponseJson.error("参数异常", null);
        }
        return couponService.productPage(couponId, userId, productName, pageNum, pageSize);
    }*/

    @ApiOperation("根据商品id获取相关优惠券")
    @ApiImplicitParams({
            @ApiImplicitParam(required = true, name = "userId", value = "用户id"),
            @ApiImplicitParam(required = true, name = "productIds", value = "商品id,以逗号隔开")
    })
    @GetMapping("/productIds")
    public ResponseJson<Map<String, Object>> couponsByProductIds(Integer userId, String productIds) {
        if (null == userId || StringUtils.isEmpty(productIds)) {
            return ResponseJson.error("参数异常", null);
        }
        return couponService.couponsByProductIds(userId, productIds);
    }

    @ApiOperation("首页优惠券列表")
    @ApiImplicitParam(required = false, name = "userId", value = "机构用户id")
    @GetMapping("/home")
    public ResponseJson<List<CouponVo>> homeCouponList(Integer userId) {
        return couponService.homeCouponList(userId);
    }

    @ApiOperation("分享券广告弹窗")
    @GetMapping("/share/display")
    public ResponseJson<String> shareCouponDisplay() {
        return couponService.shareCouponDisplay();
    }

    @ApiOperation("分享券金额展示")
    @GetMapping("/share/amount/display")
    public ResponseJson<Map<String,Object>> shareCouponAmount() {
        return couponService.shareCouponAmount();
    }
}
