package com.caimei.controller;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.caimei.components.WeChatService;
import com.caimei.model.ResponseJson;
import com.caimei.service.CouponService;
import com.caimei.util.HttpRequest;
import com.caimei.util.MessageUtil;
import com.caimei.util.OkHttpUtil;
import com.caimei.util.message.MessageModel;
import com.caimei.util.message.MqInfo;
import com.caimei.util.message.WechatTemplateMessage;
import com.caimei.util.message.enums.MessageType;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.DateUtils;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.protocol.HttpContext;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import redis.clients.jedis.ZParams;

import java.net.URI;
import java.util.ArrayList;
import java.util.Date;
import java.util.Map;

/**
 * 颜选美学商城用户API
 *
 * @author : plf
 * @date : 2021/4/21
 */
@Api(tags = "颜选美学商城用户API")
@RestController
@RequiredArgsConstructor
@RequestMapping("/user/he")
public class HeHeApi {
    @Value("${caimei.cloudApi}")
    private String cloudApi;

    @Autowired
    private WeChatService weChatService;


    @ApiOperation("微信授权登录")
    @GetMapping("/authorization")
    public ResponseJson<String> authorization(String code) throws Exception {
        String url = cloudApi + "/user/he/authorization?code=" + code;
        String parameters = HttpRequest.sendGet(url);
        return getResponseJson(parameters);
    }

    @ApiOperation("手机号验证码登录")
    @PostMapping("/mobile/login")
    public ResponseJson<String> mobileLogin(@RequestBody Map<String, Object> param) throws Exception {
        String url = cloudApi + "/user/he/mobile/login";
        String parameters = HttpRequest.sendPost(url, param);
        return getResponseJson(parameters);
    }

    @ApiOperation("发送手机号验证码")
    @GetMapping("/send")
    public ResponseJson<String> sendVerificationCode(String mobile) throws Exception {
        String url = cloudApi + "/user/he/send?mobile=" + mobile;
        String parameters = HttpRequest.sendGet(url);
        return getResponseJson(parameters);
    }

    @ApiOperation("获取首页微信群二维码")
    @GetMapping("/group/qrCode")
    public String getGroupQrCode() throws Exception {
        String url = cloudApi + "/user/he/group/qrCode";
        return HttpRequest.sendGet(url);
    }

    @ApiOperation("获取token")
    @GetMapping("/access/token")
    public ResponseJson<String> getAccessToken() throws Exception {
        String accessToken = weChatService.getAccessToken();
        return ResponseJson.success(accessToken);
    }


    @ApiOperation("获取小程序二维码图片")
    @PostMapping("/wxacode")
    public ResponseJson<String> generateWxacode(@RequestBody String params) throws Exception {
        String accessToken = weChatService.getAccessToken();
        ResponseJson<String> result = weChatService.generateWxacode(accessToken, params);
        return result;
    }

    private ResponseJson<String> getResponseJson(String parameters) {
        JSONObject object = JSONObject.parseObject(parameters);
        Integer code = object.getInteger("code");
        String msg = object.getString("msg");
        String data = object.getString("data");
        return ResponseJson.success(code, msg, data);
    }

}
