package com.caimei.mapper;

import com.caimei.model.po.CouponPo;
import com.caimei.model.po.CouponSharePo;
import com.caimei.model.po.ReceiveCouponPo;
import com.caimei.model.vo.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.springframework.web.bind.annotation.PostMapping;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * Description
 *
 * @author : Aslee
 * @date : 2021/10/26
 */
@Mapper
public interface CouponMapper {

    /**
     * 优惠券类型
     * @param couponType    劵类型 1活动券 2专享券 3新人券 4好友分享券 5好友消费券 6消费分享券
     * @return
     */
    Integer getCouponCount(int couponType);

    /**
     * 获取进行中的优惠券Id
     * @param couponType    劵类型 1活动券 2专享券 3新人券 4好友分享券 5好友消费券 6消费分享券
     * @return
     */
    List<Integer> getCurrentCouponIds(int couponType);


    /**
     * 校验该用户的领取期限是否到期
     *
     * @param couponId
     * @param userId
     * @return
     */
    boolean checkReceiveFlag(@Param("couponId") Integer couponId, @Param("userId") Integer userId);

    /**
     * 查询用户的注册时间
     * @param userId
     * @return
     */
    Date getUserRegisterTime(Integer userId);

    /**
     * 首页活动券券展示记录
     *
     * @param couponId 优惠券id
     * @param userId   用户id
     * @return
     */
    int getDisplayRecordCount(@Param("couponId") Integer couponId, @Param("userId") Integer userId);

    /**
     * 查询可领取的优惠券列表
     *
     * @param userId       用户id
     * @param productIdArr
     * @param registerTime
     * @return
     */
    List<CouponVo> findCouponList(@Param("userId") Integer userId, @Param("productIdArr") String[] productIdArr, @Param("registerTime") Date registerTime, @Param("couponType") Integer couponType);

    /**
     * 查询已领取的优惠券列表
     *
     * @param userId
     * @param productIdArr
     * @param receiveCouponId
     * @param status
     * @return
     */
    List<CouponVo> findReceiveCouponList(@Param("userId") Integer userId, @Param("productIdArr") String[] productIdArr, @Param("receiveCouponId") Integer receiveCouponId, @Param("status") Integer status);

    /**
     * 查询已领取的优惠券数量
     *
     * @param userId
     * @param status
     * @return
     */
    Integer findCouponNum(Integer userId, int status);

    /**
     * 查询优惠券商品
     * @param couponId  优惠券id
     * @param productName
     * @return
     */
    List<ProductVo> getCouponProduct(Integer couponId, String productName);

    /**
     * 查询优惠券所有商品id
     * @param couponId
     * @return
     */
    String getCouponProductIds(Integer couponId);

    /**
     * 保存优惠券弹窗记录
     * @param couponId
     * @param userId
     */
    void insertDisplayRecord(@Param("couponId") Integer couponId, @Param("userId") Integer userId);

    /**
     * 根据用户id和优惠券id查询已领取优惠券id
     *
     * @param userId
     * @param couponId
     * @return
     */
    Integer findReceiveCouponId(@Param("userId") Integer userId, @Param("couponId") Integer couponId, @Param("couponShareId") Integer couponShareId);

    /**
     * 用户领取优惠券保存
     * @param receiveCoupon
     */
    void insertReceiveCoupon(ReceiveCouponPo receiveCoupon);

    /**
     * 根据被分享者用户id查询优惠券分享记录Id
     * @param receiveUserId
     * @return
     */
    CouponSharePo getCouponShareId(Integer receiveUserId);

    /**
     * 保存消费券发送记录
     * @param couponSharePo
     */
    void insertCouponShare(CouponSharePo couponSharePo);

    /**
     * 根据优惠券id查询优惠券金额
     * @param couponId
     * @return
     */
    BigDecimal getCouponAmount(Integer couponId);

    /**
     * 获取用户手机号
     * @param shareUserId
     * @return
     */
    String getUserMobile(Integer shareUserId);

    void updateSmsSendCount(@Param("markId") int markId, @Param("count") int count);

    /**
     * 删除用户已删除的优惠券
     */
    void deleteReceiveCoupon(Integer receiveCouponId);

    /**
     * 根据id查询已领取的优惠券信息
     * @param receiveCouponId
     * @return
     */
    ReceiveCouponPo getReceiveCouponById(Integer receiveCouponId);

    Integer getShareRecordCount(Integer orderId);

    List<Double> getCouponAmountList(int couponType);
}
