package com.caimei.mapper;

import com.caimei.model.po.*;
import com.caimei.model.vo.AddressVo;
import com.caimei.model.vo.CartProductVo;
import com.caimei.model.vo.ReductionVo;
import com.caimei.model.vo.ShopVo;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/4/25
 */
@Mapper
public interface OrderSubmitMapper {
    /**
     * 根据商品集合查询供应商信息
     *
     * @param cartIds
     * @return
     */
    List<ShopVo> findShopByCartIds(@Param("cartIds") String[] cartIds);

    /**
     * 查询购物车商品信息
     *
     * @param shopId
     * @param cartIds
     * @return
     */
    List<CartProductVo> findByShopCartProduct(@Param("shopId") Integer shopId, @Param("cartIds") String[] cartIds, @Param("splitCode") String splitCode);

    /**
     * 查询用户信息
     *
     * @param userId
     * @return
     */
    UserPo findUser(Integer userId);

    /**
     * 查询商品信息
     *
     * @param skuId
     * @return
     */
    CmHeHeProductPo getProduct(Integer skuId);

    /**
     * 删除购物车
     *
     * @param userId    用户id
     * @param skuId     skuId
     * @param heUserId  分销者id
     */
    void deleteCartByProductId(@Param("userId") Integer userId, @Param("skuId") Integer skuId, @Param("heUserId") Integer heUserId);

    /**
     * 保存订单信息
     *
     * @param order
     */
    void insertOrder(CmOrderPo order);

    /**
     * 查询子订单编号
     *
     * @param orderId
     * @return
     */
    String findMaxShopOrderNo(Long orderId);

    /**
     * 保存子订单信息
     *
     * @param shopOrder
     */
    void insertShopOrder(CmShopOrderPo shopOrder);

    /**
     * 保存订单商品信息
     *
     * @param orderProduct
     */
    void insertOrderProduct(CmOrderProductPo orderProduct);

    /**
     * 保存订单阶梯信息
     *
     * @param ladderPrice
     */
    void insertOrderProductLadderPrice(OrderProductLadderPricePo ladderPrice);

    /**
     * 更新订单信息
     *
     * @param order
     */
    void updateOrder(CmOrderPo order);

    /**
     * 查询地址信息
     *
     * @param addressId
     * @return
     */
    AddressVo findByAddressId(Integer addressId);

    /**
     * 保存订单地址信息
     *
     * @param userInfo
     */
    void insertUserInfo(BpOrderUserInfoPo userInfo);

    /**
     * 根据商品id查询供应商信息
     *
     * @param skuId
     * @return
     */
    ShopVo findShopByProductId(Integer skuId);

    /**
     * 查询商品信息
     *
     * @param productId
     * @return
     */
    CartProductVo findProductById(Integer productId);

    /**
     * 查询购物车所有商品
     * @param cartId    购物车id数组
     * @return
     */
    List<CartProductVo> findCartProducts(String[] cartId);

    /**
     * 保存优惠券使用状态
     * @param receiveCouponId
     * @param orderID
     */
    void updateReceiveCouponStatus(@Param("receiveCouponId") Integer receiveCouponId, @Param("orderId") Long orderID);

    /**
     * 保存订单优惠记录
     * @param orderRecord
     */
    void insertCouponOrderRecord(CouponOrderRecordPo orderRecord);

    /**
     * 查找当前分享减免活动
     * @return
     */
    ReductionVo findCurrentReduction();

    /**
     *查询已分享次数
     * @return
     */
    Integer findShareNum(@Param("userId") Integer userId, @Param("reductionId") Integer reductionId);

    /**
     * 保存分享记录
     * @return
     */
    void insertReductionUser(ReductionUserPo reductionUserPo);

    /**
     * 查找分享减免记录
     * @return
     */
    ReductionVo findReduction(Integer reductionUserId);

    /**
     * 分享减免关联订单id
     * @return
     */
    void updateReductionUserId(@Param("reductionUserId") Integer reductionUserId,@Param("orderId") Long orderId);

    String getHelpContent(Integer helpId);


}
