package com.caimei.service;

import com.caimei.model.ResponseJson;
import com.caimei.model.dto.CouponsDto;
import com.caimei.model.vo.CouponVo;
import com.caimei.model.vo.ProductVo;
import com.github.pagehelper.PageInfo;

import java.util.List;
import java.util.Map;

/**
 * Description
 *
 * @author : Aslee
 * @date : 2021/10/26
 */
public interface CouponService {
    /**
     * 首页优惠券弹窗
     * @param userId    用户id
     * @return
     */
    ResponseJson<Map<String, Integer>> couponDisplay(Integer userId);

    /**
     * 领券中心
     *
     * @param userId   机构用户id
     * @param pageNum  页码
     * @param pageSize 每页数量
     * @return
     */
    ResponseJson<PageInfo<CouponVo>> couponCenter(Integer userId, int pageNum, int pageSize);

    /**
     * 已领取优惠券中心
     *
     * @param userId   用户id
     * @param status   使用状态: 1未使用 2已使用 3已失效
     * @param pageNum  页码
     * @param pageSize 每页数量
     * @return
     */
    ResponseJson<PageInfo<CouponVo>> receiveCouponList(Integer userId, Integer status, int pageNum, int pageSize);

    /**
     * 统计已领取优惠券数量
     * @param userId    用户id
     * @return
     */
    ResponseJson<Map<String, Integer>> receiveCount(Integer userId);

    /**
     * 优惠券商品数据
     * @param couponId  优惠券id
     * @param userId    用户id
     * @param productName
     * @param pageNum   页码
     * @param pageSize  每页数量
     * @return
     */
//    ResponseJson<PageInfo<ProductVo>> productPage(Integer couponId, Integer userId, String productName, int pageNum, int pageSize);

    /**
     * 领取优惠券
     * @param couponsDto
     * @return
     */
    ResponseJson<String> collarCoupons(CouponsDto couponsDto);


    ResponseJson<List<CouponVo>> productDetailCoupons(Integer productId, Integer userId);

    /**
     * 根据商品id查询相关优惠券
     * @param userId        用户id
     * @param productIds    商品id,以逗号隔开
     * @return
     */
    ResponseJson<Map<String, Object>> couponsByProductIds(Integer userId, String productIds);

    /**
     * 首页优惠券列表
     */
    ResponseJson<List<CouponVo>> homeCouponList(Integer userId);

    /**
     * 分享券广告展示
     */
    ResponseJson<String> shareCouponDisplay();

    /**
     * 分享券金额展示
     * @return
     */
    ResponseJson<Map<String, Object>> shareCouponAmount();
}
