package com.caimei.service.impl;

import com.caimei.mapper.CouponMapper;
import com.caimei.model.ResponseJson;
import com.caimei.model.dto.CouponsDto;
import com.caimei.model.po.ReceiveCouponPo;
import com.caimei.model.vo.*;
import com.caimei.service.CouponService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * Description
 *
 * @author : Aslee
 * @date : 2021/10/26
 */
@Service
public class CouponServiceImpl implements CouponService {
    @Resource
    private CouponMapper couponMapper;


    @Override
    public ResponseJson<Map<String, Integer>> couponDisplay(Integer userId) {
        HashMap<String, Integer> map = new HashMap<>(2);
        if (null == userId) {
            // 查询进行中的新人券数量
            Integer newUserCouponCount = couponMapper.getCouponCount(3);
            // 查询进行中的活动券数量
            Integer activityCouponCount = couponMapper.getCouponCount(1);
            map.put("newUserCouponFlag", newUserCouponCount > 0 ? 1 : 0);
            map.put("actCouponFlag", activityCouponCount > 0 ? 1 : 0);
        } else {
            // 是否展示新人券弹窗
            boolean newUserDisplayFlag = checkDisplayFlag(userId ,3);
            // 是否展示活动券弹窗
            boolean actDisplayFlag = checkDisplayFlag(userId, 1);
            // 是否有可展示的专享券
            boolean singleCouponFlag = checkDisplayFlag(userId, 2);
            // 是否有可展示的好友分享券
            boolean shareCouponFlag = checkDisplayFlag(userId, 4);
            // 是否有可展示的好友消费券
            boolean consumeCouponFlag = checkDisplayFlag(userId, 5);
            map.put("newUserCouponFlag", newUserDisplayFlag ? 1 : 0);
            map.put("actCouponFlag", actDisplayFlag ? 1 : 0);
            map.put("otherCouponFlag", singleCouponFlag || shareCouponFlag || consumeCouponFlag ? 1 : 0);
        }
        return ResponseJson.success(map);
    }

    private boolean checkDisplayFlag(Integer userId, Integer couponType) {
        // 查询该用户的注册时间
        Date registerTime = couponMapper.getUserRegisterTime(userId);
        // 查询用户可领取该类型的优惠券
        List<CouponVo> couponList = couponMapper.findCouponList(userId, null, registerTime, couponType);
        boolean displayFlag = false;
        CouponVo coupon = null;
        if (couponList != null && couponList.size() > 0) {
            coupon = couponList.get(0);
        }
        if (null == coupon) {
            return false;
        }
        // 查询是否已经对该用户弹窗展示过
        int displayCount = couponMapper.getDisplayRecordCount(coupon.getCouponId(), userId);
        if (displayCount <= 0) {
            couponMapper.insertDisplayRecord(coupon.getCouponId(), userId);
            displayFlag = true;
        }
        return displayFlag;
    }

    @Override
    public ResponseJson<PageInfo<CouponVo>> couponCenter(Integer userId, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        Date registerTime = couponMapper.getUserRegisterTime(userId);
        List<CouponVo> couponList = couponMapper.findCouponList(userId, null, registerTime, null);
        PageInfo<CouponVo> pageInfo = new PageInfo<>(couponList);
        return ResponseJson.success(pageInfo);
    }

    @Override
    public ResponseJson<PageInfo<CouponVo>> receiveCouponList(Integer userId, Integer status, int pageNum, int pageSize) {
        PageHelper.startPage(pageNum, pageSize);
        List<CouponVo> couponList = couponMapper.findReceiveCouponList(userId, null, null, status);
        couponList.forEach(coupon -> coupon.setUseStatus(status));
        PageInfo<CouponVo> pageInfo = new PageInfo<>(couponList);
        return ResponseJson.success(pageInfo);
    }

    @Override
    public ResponseJson<Map<String, Integer>> receiveCount(Integer userId) {
        Map<String, Integer> map = new HashMap<>(3);
        Integer unusedNum = couponMapper.findCouponNum(userId, 1);
        Integer usedNum = couponMapper.findCouponNum(userId, 2);
        Integer expiredNum = couponMapper.findCouponNum(userId, 3);
        map.put("unusedNum", unusedNum);
        map.put("usedNum", usedNum);
        map.put("expiredNum", expiredNum);
        return ResponseJson.success(map);
    }

    @Override
    public ResponseJson<String> collarCoupons(CouponsDto couponsDto) {
        Integer receiveCouponId = couponMapper.findReceiveCouponId(couponsDto.getUserId(), couponsDto.getCouponId(), couponsDto.getCouponShareId());
        if (receiveCouponId != null && receiveCouponId > 0) {
            return ResponseJson.error("已经领取过了", null);
        }
        ReceiveCouponPo receiveCoupon = new ReceiveCouponPo();
        BeanUtils.copyProperties(couponsDto, receiveCoupon);
        receiveCoupon.setSource(1);
        receiveCoupon.setStatus(1);
        receiveCoupon.setReceiveTime(new Date());
        receiveCoupon.setDelFlag(0);
        couponMapper.insertReceiveCoupon(receiveCoupon);
        return ResponseJson.success("领取成功");
    }

    @Override
    public ResponseJson<List<CouponVo>> productDetailCoupons(Integer productId, Integer userId) {
        Date registerTime = couponMapper.getUserRegisterTime(userId);
        // 所有优惠券列表
        String[] productArr = new String[1];
        productArr[0] = productId + "";
        // 可领取优惠券列表
        List<CouponVo> couponList = couponMapper.findCouponList(userId, productArr, registerTime,null);
        List<CouponVo> totalCouponList = new ArrayList<>(couponList);
        // 已领取未使用优惠券列表
        List<CouponVo> receiveCouponList;
        if (null != userId && userId > 0) {
            receiveCouponList = couponMapper.findReceiveCouponList(userId, productArr, null, 1);
            if (null != receiveCouponList) {
                receiveCouponList.forEach(receiveCoupon -> receiveCoupon.setUseStatus(1));
                totalCouponList.addAll(receiveCouponList);
            }
        }
        if (totalCouponList.size() > 0) {
            totalCouponList.sort(Comparator.comparing(CouponVo::getCreateDate));
        }
        return ResponseJson.success(totalCouponList);
    }

    @Override
    public ResponseJson<Map<String, Object>> couponsByProductIds(Integer userId, String productIds) {
        Map<String,Object> couponsMap = new HashMap<>(2);
        String[] productIdArr = productIds.split(",");
        Date registerTime = couponMapper.getUserRegisterTime(userId);
        // 包含productIds的可领取优惠券列表
        List<CouponVo> ableCouponList = couponMapper.findCouponList(userId, productIdArr, registerTime, null);
        // 包含productIds的已领取优惠券列表
        List<CouponVo> receiveCouponList = couponMapper.findReceiveCouponList(userId, productIdArr, null, 1);
        receiveCouponList.forEach(coupon -> coupon.setUseStatus(1));
        couponsMap.put("ableCouponList", ableCouponList);
        couponsMap.put("receiveCouponList", receiveCouponList);
        return ResponseJson.success(couponsMap);
    }

    @Override
    public ResponseJson<List<CouponVo>> homeCouponList(Integer userId) {
        Date registerTime = couponMapper.getUserRegisterTime(userId);
        List<CouponVo> couponList = couponMapper.findCouponList(userId, null, registerTime, null);
        return ResponseJson.success(couponList);
    }

    @Override
    public ResponseJson<String> shareCouponDisplay() {
        Integer inviteCouponCount = couponMapper.getCouponCount(4);
        Integer consumeCouponCount = couponMapper.getCouponCount(5);
        Integer shareCouponCount = couponMapper.getCouponCount(6);
        String couponDisplay = "";
        couponDisplay += inviteCouponCount > 0 ? "1" : "";
        couponDisplay += consumeCouponCount > 0 ? ",2" : "";
        couponDisplay += shareCouponCount > 0 ? ",3" : "";
        return ResponseJson.success(couponDisplay);
    }

    @Override
    public ResponseJson<Map<String, Object>> shareCouponAmount() {
        // 好友邀请券
        List<Double> inviteCouponList = couponMapper.getCouponAmountList(4);
        // 好友消费券
        List<Double> consumeCouponList = couponMapper.getCouponAmountList(5);
        // 消费分享券
        List<Double> shareCouponList = couponMapper.getCouponAmountList(6);
        HashMap<String, Object> map = new HashMap<>();
        map.put("inviteCouponList", inviteCouponList);
        map.put("consumeCouponList", consumeCouponList);
        map.put("shareCouponList", shareCouponList);
        return ResponseJson.success(map);
    }

}
