package com.caimei.service.impl;

import com.caimei.mapper.*;
import com.caimei.model.ResponseJson;
import com.caimei.model.po.*;
import com.caimei.model.vo.*;
import com.caimei.service.OrderSubmitService;
import com.caimei.service.ShoppingCartService;
import com.caimei.util.MathUtil;
import com.caimei.util.MessageUtil;
import com.caimei.util.OrderNoUtils;
import com.caimei.util.ProductUtils;
import com.caimei.util.message.InsideMessage;
import com.caimei.util.message.MessageModel;
import com.caimei.util.message.MqInfo;
import com.caimei.util.message.WechatTemplateMessage;
import com.caimei.util.message.enums.MessageType;
import com.google.common.util.concurrent.AtomicDouble;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.client.utils.DateUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/4/25
 */
@Slf4j
@Service
public class OrderSubmitServiceImpl implements OrderSubmitService {
    @Resource
    private OrderSubmitMapper orderSubmitMapper;
    @Resource
    private OrderMapper orderMapper;
    @Resource
    private ShoppingCartService shoppingCartService;
    @Resource
    private ProductMapper productMapper;
    @Resource
    private CouponMapper couponMapper;
    @Resource
    private CollageMapper collageMapper;
    @Value("${caimei.oldapi}")
    private String domain;

    @Override
    public ResponseJson<Map<String, Object>> orderConfirm(Integer userId, String cartIds, Integer skuId, Integer productId, Integer productCount, Integer heUserId, Integer couponId, Integer collageFlag) {
        Map<String, Object> confirmData = new HashMap<>(5);
        log.info("<<<<< 结算订单 >>>>>");
        //商品总金额
        AtomicReference<BigDecimal> totalPrice = new AtomicReference<>(BigDecimal.ZERO);
        List<CartProductVo> totalProducts = new ArrayList<>();
        List<ShopVo> shopList = new ArrayList<>();
        List<String> productIdList = new ArrayList<>();
        Integer onlinePay = 0;
        if (skuId != null && skuId > 0) {
            //立即购买
            ShopVo shop = orderSubmitMapper.findShopByProductId(skuId);
            if (StringUtils.isEmpty(shop.getSplitCode())) {
                // 商品未填写分账号，不支持线上支付
                onlinePay = -1;
            }
            shopList.add(shop);
        } else {
            String[] cartId = cartIds.split(",");
            shopList = orderSubmitMapper.findShopByCartIds(cartId);
            for (ShopVo shop : shopList) {
                if (StringUtils.isEmpty(shop.getSplitCode())) {
                    // 商品未填写分账号，不支持线上支付
                    onlinePay = -2;
                }
            }
        }
        String helpContent=null;
        boolean flag=false;
        for (int i=0;i<shopList.size();i++) {
            //订单中存在不能退货的商品则给出不能退货提示
            if(null!=shopList.get(i).getReturnGoodsStutas()&&shopList.get(i).getReturnGoodsStutas()==2){
                flag=true;
                helpContent = orderSubmitMapper.getHelpContent(1040);
                break;
            }
        }
        if(flag){
            confirmData.put("returnGoodsStutas", 2);
            confirmData.put("helpContent", helpContent);
        }else{
            confirmData.put("returnGoodsStutas", 1);
            confirmData.put("helpContent", null);
        }
        shopList.forEach(shop -> {
            shop.setLogo(ProductUtils.getImageURL("shopLogo", shop.getLogo(), 0, domain));
            List<CartProductVo> shopProducts = new ArrayList<>();
            if (skuId != null && skuId > 0) {
                CartProductVo cartProduct = orderSubmitMapper.findProductById(skuId);
                cartProduct.setNum(productCount);
                cartProduct.setHeUserId(heUserId);
                shopProducts.add(cartProduct);
            } else {
                String[] cartId = cartIds.split(",");
                shopProducts = orderSubmitMapper.findByShopCartProduct(shop.getShopId(), cartId, shop.getSplitCode());
            }
            BigDecimal shopTotalPrice = BigDecimal.ZERO;
            for (CartProductVo product : shopProducts) {
                shoppingCartService.setPrice(product, userId, collageFlag);
                productIdList.add(product.getProductId().toString());
                shopTotalPrice = MathUtil.add(shopTotalPrice, MathUtil.mul(product.getNum(), product.getPrice()));
            }
            shop.setShopTotalPrice(shopTotalPrice);
            shop.setProductList(shopProducts);
            totalProducts.addAll(shopProducts);
            totalPrice.set(MathUtil.add(totalPrice.get(), shopTotalPrice));
        });
        String[] productIdArr = productIdList.toArray(new String[0]);
        // 已领取商品可用优惠券
        List<CouponVo> receiveCouponList = couponMapper.findReceiveCouponList(userId, productIdArr, null, 1);
        if (null != couponId) {
            // 领券购买对应优惠券领取标识
            AtomicBoolean couponReceiveFlag = new AtomicBoolean(false);
            receiveCouponList.forEach(coupon -> {
                if (couponId.equals(coupon.getCouponId())) {
                    couponReceiveFlag.set(true);
                }
            });
            if (!couponReceiveFlag.get()) {
                // 未领取，自动领取
                ReceiveCouponPo receiveCoupon = new ReceiveCouponPo();
                receiveCoupon.setCouponId(couponId);
                receiveCoupon.setUserId(userId);
                receiveCoupon.setSource(3);
                receiveCoupon.setStatus(1);
                receiveCoupon.setReceiveTime(new Date());
                receiveCoupon.setDelFlag(0);
                couponMapper.insertReceiveCoupon(receiveCoupon);
                // 重新查询已领取商品可用优惠券
                receiveCouponList = couponMapper.findReceiveCouponList(userId, productIdArr, null, 1);
                // 再将该领取的优惠券删除，未提交订单不算真正领取
                couponMapper.deleteReceiveCoupon(receiveCoupon.getReceiveCouponId());
            }
        }
        /*ListIterator<CouponVo> iterator = receiveCouponList.listIterator();
        while (iterator.hasNext()) {
            CouponVo coupon = iterator.next();
            // 能参与该优惠券的总金额
            BigDecimal touchCouponAmount = BigDecimal.ZERO;
            // 是否包含优惠券可用商品
            boolean useFlag = false;
            if (coupon.getProductType() == 1) {
                //全商城商品
                touchCouponAmount = totalPrice.get();
                useFlag = true;
            } else {
                // 参与优惠券的所有商品id
                String couponProductIds = couponMapper.getCouponProductIds(coupon.getCouponId());
                if (productId != null && productId > 0) {
                    // 立即购买
                    CartProductVo cartProduct = totalProducts.get(0);
                    if (couponProductIds.contains(cartProduct.getProductId().toString())) {
                        useFlag = true;
                        touchCouponAmount = totalPrice.get();
                    }
                } else {
                    // 购物车结算
                    for (CartProductVo product : totalProducts) {
                        if (couponProductIds.contains(product.getProductId().toString())) {
                            useFlag = true;
                            touchCouponAmount = MathUtil.add(touchCouponAmount, MathUtil.mul(product.getNum(), product.getPrice()));
                        }
                    }
                }
            }
            if ((useFlag && coupon.getNoThresholdFlag() == 1) || touchCouponAmount.compareTo(coupon.getTouchPrice()) > 0) {
                // 满足优惠条件
                coupon.setTouchFlag(1);
            } else {
                coupon.setTouchFlag(0);
            }
            if (!useFlag) {
                iterator.remove();
            }
        }*/
        receiveCouponList.sort((o1, o2) -> {
            // 按满足优惠条件/优惠金额进行排序
            /*if (!o1.getTouchFlag().equals(o2.getTouchFlag())) {
                return o1.getTouchFlag() - o2.getTouchFlag();
            } else {*/
            return o2.getCouponAmount().compareTo(o1.getCouponAmount());
//            }
        });
        // 可参与的分享减免活动
        ReductionVo currentReduction = orderSubmitMapper.findCurrentReduction();
        confirmData.put("reduction", null);
        if (null != currentReduction) {
            // 查询已分享次数
            Integer shareNum = orderSubmitMapper.findShareNum(userId, currentReduction.getReductionId());
            if ((null == collageFlag || 0 == collageFlag) && shareNum < currentReduction.getShareNum() && MathUtil.compare(totalPrice.get(), currentReduction.getTouchPrice()) >= 0) {
                // 有剩余分享次数且订单金额大于等于分享条件金额
                confirmData.put("reduction", currentReduction);
            }
        }
        confirmData.put("onlinePay", onlinePay);
        confirmData.put("receiveCouponList", receiveCouponList);
        confirmData.put("totalPrice", totalPrice);
        confirmData.put("shopList", shopList);
        return ResponseJson.success(confirmData);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseJson<Map<String, String>> orderSubmit(Integer userId, Integer cartType, Integer addressId, Map<String, Object> discountInfo, List<Map<String, Object>> orderInfo, Map<String, Object> payInfo) {
        /*
         * 逻辑处理 start
         */
        log.info("******************** 提交订单逻辑处理 start *******************");
        //机构用户
        UserPo user = orderSubmitMapper.findUser(userId);
        if (null == user) {
            return ResponseJson.error("用户信息异常", null);
        }
        // 运费
        BigDecimal freight = BigDecimal.ZERO;
        // 商品总数量
        Integer productCount = 0;
        // 商品总金额 （商品单价乘以数量，再加上税费[默认0]）
        BigDecimal productTotalFee = BigDecimal.ZERO;
        // 小计金额（商品折后单价乘以数量，再加上税费[默认0])
        BigDecimal orderTotalFee = BigDecimal.ZERO;
        // 订单总额（小计金额减去经理折扣后，再加上运费[默认0])
        BigDecimal payTotalFee = MathUtil.add(BigDecimal.ZERO,freight);
        // 真实支付金额（订单总额减去抵扣的账户余额） 实际支付金额(商品金额+运费-余额抵扣)
        BigDecimal payableAmount =  MathUtil.add(BigDecimal.ZERO,freight);
        // 是否能线上支付
        AtomicBoolean onlinePay = new AtomicBoolean(true);

        /*
         * 初始化主订单参数
         */
        CmOrderPo order = new CmOrderPo();
        String curDateStr = (new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")).format(new Date());
        // 订单号
        String orderNo = null;
        orderNo = OrderNoUtils.getOrderNo("H");
        order.setOrderNo(orderNo);
        // 运营人员下单
        order.setBuyUserID(user.getUserID());
        order.setOrderType(2);
        order.setOrderSubmitType(0);
        order.setConfirmFlag("2");
        order.setUserID(user.getUserID().longValue());
        // 订单来源
        order.setOrderSource("6");
        order.setOrganizeID(0);
        order.setUpdateDate(curDateStr);
        order.setPayFlag("0");
        order.setOnlinePayFlag("0");
        order.setPreferential(BigDecimal.ZERO);
        order.setDiscountFee(BigDecimal.ZERO);
        // 订单提交时间
        order.setOrderTime(curDateStr);
        // 默认订单可以拆分
        order.setSplitFlag("1");
        // 发票类型
        order.setInvoiceFlag("0");
        order.setReceiptStatus("1");
        order.setPayStatus("1");
        order.setZeroCostFlag(0);
        order.setSendOutStatus("1");
        order.setRefundType("0");
        // 是否包含活动商品(受订单未支付自动关闭时间影响)  0 否 1 是
        order.setHasActProduct("0");
        // 订单状态 0 有效  其它无效
        order.setDelFlag("0");
        // 是否确认付款供应商
        order.setAffirmPaymentFlag("0");
        /*
         * 订单商品
         */
        List<CmOrderProductPo> orderProductList = new ArrayList<>();
        Integer collageFlag = (Integer) discountInfo.get("collageFlag");
        Integer collageId = (Integer) discountInfo.get("collageId");
        for (Map<String, Object> shopOrderInfo : orderInfo) {
            Integer shopId = (Integer) shopOrderInfo.get("shopId");
            if (null == shopId) {
                return ResponseJson.error("供应商数据异常", null);
            }
            // 一个子订单对应的商品信息
            List<Map<String, Integer>> productInfoList = (List<Map<String, Integer>>) shopOrderInfo.get("productInfo");
            if (null == productInfoList) {
                return ResponseJson.error(-1, "订单商品数据异常", null);
            }
            /*
             * 整理订单商品信息
             */
            if (!CollectionUtils.isEmpty(productInfoList)) {
                // 遍历所有商品
                for (Map<String, Integer> productTemp : productInfoList) {
                    Integer skuId = productTemp.get("skuId");
                    Integer productNum = productTemp.get("productNum");
                    Integer heUserId = productTemp.get("heUserId");
                    heUserId = heUserId == null ? 0 : heUserId;
                    // 统计商品总数量
                    productCount += productNum;

                    // 获取商品信息
                    CmHeHeProductPo product = orderSubmitMapper.getProduct(skuId);
                    if (null == product) {
                        return ResponseJson.error("订单商品数据异常", null);
                    }
                    if (null == productNum || productNum == 0) {
                        return ResponseJson.error("商品购买数量异常", null);
                    }
                    // 获取商品购买价格(活动阶梯价格)
                    BigDecimal productPrice = product.getPrice();
                    // 成本价
                    BigDecimal costPrice = BigDecimal.ZERO;
                    // 机构税费(单)
                    BigDecimal addedValueTax = new BigDecimal(0);
                    int priceType = 0;
                    // 商品参与的活动id
                    Integer activityId = productMapper.getActivityIdBySkuId(skuId);
                    // 商品参与的限时特价活动
                    BigDecimal discountPrice = productMapper.getDiscountPrice(skuId);
                    // 内部优惠折扣
                    HeHeDiscountVo discount = productMapper.findProductDiscount(product.getProductId(), userId);
                    if (activityId != null && activityId > 0) {
                        //活动阶梯
                        List<ActivityLadderVo> ladderList = productMapper.getActivityLadderList(activityId, skuId);
                        if (ladderList != null && ladderList.size() > 0) {
                            priceType = 1;
                            for (ActivityLadderVo ladder : ladderList) {
                                if (productNum >= ladder.getBuyNum()) {
                                    productPrice = ladder.getBuyPrice();
                                }
                            }
                        }
                    } else if (collageFlag != null && 1 == collageFlag) {
                        // 拼团价
                        CmHeheCollageProductPo collageProduct = productMapper.findCollageProduct(skuId);
                        if (collageProduct != null) {
                            priceType = 2;
                            productPrice = collageProduct.getPrice();
                        }
                    } else if (discountPrice != null) {
                        // 限时特价
                        priceType = 3;
                        productPrice = discountPrice;
                    } else if (discount != null) {
                        // 内部优惠价
                        priceType = 4;
                        // 折扣价
                        BigDecimal disPrice = discount.getDiscountPrice();
                        // 折扣率
                        Integer dis = discount.getDiscount();
                        if (null != disPrice && disPrice.compareTo(BigDecimal.ZERO) > 0) {
                            productPrice = disPrice;
                        } else if (null != dis && dis > 0) {
                            productPrice = MathUtil.div(MathUtil.mul(product.getPrice(), dis), 100);
                        }
                    }
                    //不含税可开票商品计算税费
                    boolean addTaxFlag = (0 == product.getIncludedTax()) && (1 == product.getInvoiceType() || 2 == product.getInvoiceType());
                    if (addTaxFlag) {
                        addedValueTax = MathUtil.div(MathUtil.mul(productPrice, product.getClubTaxPoint()), BigDecimal.valueOf(100), 2);
                        productPrice = MathUtil.add(productPrice, addedValueTax);
                        BigDecimal normalPrice = product.getNormalPrice();
                        if (null != normalPrice) {
                            BigDecimal normalPriceTax = MathUtil.div(MathUtil.mul(normalPrice, product.getClubTaxPoint()), BigDecimal.valueOf(100), 2);
                            product.setNormalPrice(MathUtil.add(normalPrice, normalPriceTax));
                        }
                        product.setPrice(productPrice);
                    }
                    if (MathUtil.compare(productPrice, BigDecimal.ZERO) == 0) {
                        return ResponseJson.error("商品购买价格不能为0", null);
                    }
                    // 单个商品的金额
                    BigDecimal productFee = MathUtil.mul(productPrice, productNum);
                    // 统计商品总金额
                    productTotalFee = MathUtil.add(productTotalFee, productFee);

                    // 判断是否选中固定成本价
                    if (MathUtil.compare(product.getCostPrice(), 0) > 0 && 1 == product.getCostType()) {
                        costPrice = product.getCostPrice();
                    }
                    // 判断是否选中比例成本价
                    if (MathUtil.compare(product.getCostProportional(), 0) > 0 && 2 == product.getCostType()) {
                        // 通过售价*比例得到成本价
                        costPrice = BigDecimal.valueOf(MathUtil.div(MathUtil.mul(productPrice, product.getCostProportional()), 100, 2).floatValue());
                    }
                    product.setCostPrice(costPrice);

                    /*
                     * 整理订单商品数据
                     */
                    CmOrderProductPo orderProduct = setOrderProduct(productNum, product, priceType, productPrice, addedValueTax);
                    orderProduct.setHeUserId(heUserId);
                    // 加入订单商品列表
                    orderProductList.add(orderProduct);

                    if (cartType == 1) {
                        orderSubmitMapper.deleteCartByProductId(user.getUserID(), skuId, heUserId);
                        log.info(">>>>>>>>>>>>>>>>>>>>>>>>>>删除用户购物车数据productId：" + skuId);
                    }
                }
            }
        }

        //处理优惠券
        BigDecimal couponAmount = BigDecimal.ZERO;
        CouponVo coupon = null;
        // 已领取优惠券id
        Integer receiveCouponId = null;
        Integer couponId = (Integer) discountInfo.get("couponId");
        Integer couponShareId = (Integer) discountInfo.get("couponShareId");
        if (null != couponId && couponId > 0) {
            receiveCouponId = couponMapper.findReceiveCouponId(userId, couponId, couponShareId);
            if (null == receiveCouponId) {
                // 商品详情领券购买，提交订单时自动领券并使用
                ReceiveCouponPo receiveCoupon = new ReceiveCouponPo();
                receiveCoupon.setCouponId(couponId);
                receiveCoupon.setUserId(userId);
                receiveCoupon.setSource(3);
                receiveCoupon.setStatus(1);
                receiveCoupon.setReceiveTime(new Date());
                receiveCoupon.setDelFlag(0);
                couponMapper.insertReceiveCoupon(receiveCoupon);
                receiveCouponId = receiveCoupon.getReceiveCouponId();
            }
            List<CouponVo> receiveCouponList = couponMapper.findReceiveCouponList(userId, null, receiveCouponId, 1);
            if (receiveCouponList == null || receiveCouponList.size() <= 0) {
                // 设置手动回滚事务
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return ResponseJson.error("优惠券使用异常", null);
            } else {
                coupon = receiveCouponList.get(0);
            }
            couponAmount = coupon.getCouponAmount();
            log.info("优惠券抵扣金额：" + couponAmount);
        }
        order.setCouponAmount(couponAmount);
        // 处理分享减免
        BigDecimal reductionAmount = BigDecimal.ZERO;
        Integer reductionUserId = (Integer) discountInfo.get("reductionUserId");
        if (null != reductionUserId && 0 != reductionUserId) {
            // 分享减免活动
            ReductionVo reductionVo = orderSubmitMapper.findReduction(reductionUserId);
            if (null != reductionVo) {
                reductionAmount = reductionVo.getReducedAmount();
            } else {
                // 设置手动回滚事务
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return ResponseJson.error("分享减免异常", null);
            }
        }
        order.setReductionAmount(reductionAmount);

        // 设置是否是二手订单
        order.setSecondHandOrderFlag("0");
        order.setPromotionFullReduction(BigDecimal.ZERO);
        // 商品总数量
        order.setProductCount(productCount);
        // 赠品数量
        order.setPresentCount(0);
        //促销赠品数量
        order.setPromotionalGiftsCount(0);
        // 0包邮 -1到付 1 有运费
        order.setFreePostFlag("0");
        order.setFreight(freight);
        // 商品总额
        order.setProductTotalFee(productTotalFee);
        // 订单总额（商品金额+运费）
        payTotalFee = MathUtil.add(productTotalFee,payTotalFee);
        orderTotalFee = productTotalFee;
        order.setOrderTotalFee(orderTotalFee);
        // 优惠券抵扣
        payTotalFee = MathUtil.sub(payTotalFee, couponAmount);
        // 分享减免
        payTotalFee = MathUtil.sub(payTotalFee, reductionAmount);
        // 订单状态
        order.setStatus("11");
        if (MathUtil.compare(payTotalFee, 0) < 0 && MathUtil.compare(productTotalFee, 0) >= 0 && MathUtil.compare(couponAmount, 0) > 0) {
            // 订单金额因为使用优惠券而小于0，将金额置为0
            payTotalFee = BigDecimal.ZERO;
            order.setCouponAmount(productTotalFee);
            order.setStatus("31");
            order.setReceiptStatus("3");
        }
        order.setPayTotalFee(payTotalFee);
        order.setConfirmTime(curDateStr);

        // 余额支付金额
        order.setBalancePayFee(BigDecimal.ZERO);
        // 实际支付金额(商品金额+运费-余额抵扣)
        payableAmount = MathUtil.add(productTotalFee,payableAmount);
        order.setPayableAmount(payableAmount);
        // 售后条款
        order.setClauseID(1L);
        order.setClauseName("无条款");
        // 是否返佣订单
        order.setRebateFlag("0");
        // 判断前端传入orderShouldPayFee订单应付金额,和后台计算应付金额对比
        BigDecimal orderShouldPayFee = new BigDecimal(payInfo.get("orderShouldPayFee").toString());
        double v = MathUtil.sub(payableAmount, orderShouldPayFee).doubleValue();
        log.info(">>>>>payableAmount:" + payableAmount + " ,orderShouldPayFee:" + orderShouldPayFee);
        // 考虑前端计算不精确
        if (v < -0.1d || v > 0.1d) {
            // 设置手动回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("订单付款金额异常", null);
        }

        /**
         * 保存主订单数据
         */
        orderSubmitMapper.insertOrder(order);
        log.info(">>>>>>>>>>>>>>>>>>>>>>>>>>新增主订单(insert[cm_order])orderId：" + order.getOrderID());

        /**
         * 设置订单商品订单号
         */
        for (CmOrderProductPo orderProduct : orderProductList) {
            orderProduct.setOrderID(order.getOrderID());
            orderProduct.setOrderNo(order.getOrderNo());
            orderProduct.setOrganizeID(order.getOrganizeID());
        }

        /**
         * 整理 子订单信息
         */
        // 收集子订单供应商ID字符串
        String shopOrderIds = "";
        List<CmShopOrderPo> shopOrderList = new ArrayList<>();
        for (Map<String, Object> shopOrderInfo : orderInfo) {
            Integer shopId = (Integer) shopOrderInfo.get("shopId");
            String shopNote = (String) shopOrderInfo.get("note");
            String splitCode = (String) shopOrderInfo.get("splitCode");
            // 初始化子订单信息
            CmShopOrderPo shopOrder = initShopOrder(order, orderProductList, shopId, splitCode, shopNote);
            shopOrderList.add(shopOrder);
        }

        //分摊优惠
        BigDecimal allDiscount = MathUtil.add(couponAmount, reductionAmount);
        AtomicReference<BigDecimal> totalBrokerage = new AtomicReference<>(BigDecimal.ZERO);
        shopOrderList.forEach(shopOrder -> totalBrokerage.set(MathUtil.add(totalBrokerage, shopOrder.getBrokerage())));
        shopOrderList.sort((o1, o2) -> o2.getBrokerage().compareTo(o1.getBrokerage()));
        log.info(">>>>>>>>>>>>>>>分摊前总佣金：" + totalBrokerage);
        // 前面子订单的分摊优惠总和
        AtomicDouble allReduction = new AtomicDouble(0);
        if (MathUtil.compare(allDiscount, BigDecimal.ZERO) > 0) {
            for (int i = 0; i < shopOrderList.size(); i++) {
                if (i < shopOrderList.size() - 1) {
                    //最终支付金额=分摊前支付金额oldprice-折扣*（分摊前佣金/分摊前总佣金）
                    //优惠分摊金额
                    double discount = MathUtil.mul(allDiscount, MathUtil.div(shopOrderList.get(i).getBrokerage(), totalBrokerage.get()), 2).doubleValue();
                    shopOrderList.get(i).setEachDiscount(discount);
                    //子订单真实支付金额
                    double realPay = MathUtil.round(MathUtil.sub(shopOrderList.get(i).getNeedPayAmount(), discount), 2).doubleValue();
                    shopOrderList.get(i).setRealPay(realPay);
                    allReduction.set(MathUtil.add(discount, allReduction.get()).doubleValue());
                }
                if (i == shopOrderList.size() - 1) {
                    //最后一个子订单的分摊优惠金额=全优惠allDiscount-已分摊
                    double discount = MathUtil.sub(allDiscount, allReduction.get()).doubleValue();
                    shopOrderList.get(i).setEachDiscount(discount);
                    double realPay = MathUtil.round(MathUtil.sub(shopOrderList.get(i).getNeedPayAmount(), discount), 2).doubleValue();
                    shopOrderList.get(i).setRealPay(realPay);
                }
                //有优惠金额，最高手续费=支付金额*0.65%>10取0.65，不大于取10（合利宝最高0.65%）
                double realPay = shopOrderList.get(i).getRealPay();
                double charge = MathUtil.mul(realPay, 0.0065, 2).doubleValue();
                //佣金>最高手续费可以线上，否则线下
                double bro = MathUtil.sub(realPay, shopOrderList.get(i).getShouldPayShopAmount()).doubleValue();
                if (bro < charge) {
                    onlinePay.set(false);
                }
            }
        } else {
            //无优惠金额
            shopOrderList.forEach(so -> {
                BigDecimal realPay = so.getNeedPayAmount();
                BigDecimal charge = MathUtil.mul(realPay, 0.0065, 2);
                BigDecimal bro = so.getBrokerage();
                so.setRealPay(realPay.doubleValue());
                so.setEachDiscount(0d);
                if (MathUtil.compare(bro, charge) < 0) {
                    onlinePay.set(false);
                }
            });
        }

        if (!onlinePay.get()) {
            //设置手动回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            String msg = "";
            if (productCount > 1) {
                msg = "抱歉，部分商品支付正在调整，暂不支持下单";
            } else {
                msg = "抱歉，该商品支付正在调整，暂不支持下单";
            }
            return ResponseJson.error(-1, msg, null);
        }

        for (int i = 0; i < shopOrderList.size(); i++) {
            CmShopOrderPo shopOrder = shopOrderList.get(i);
            /*
             * 保存子订单信息到数据库
             */
            orderSubmitMapper.insertShopOrder(shopOrder);
            log.info(">>>>>>>>>>>>>>>>>>>>>>>>>>新增子订单(insert[cm_shop_order])shopOrderId：" + shopOrder.getShopOrderID());
            // 保存子订单号
            shopOrderIds += (("".equals(shopOrderIds) ? "" : ",") + shopOrder.getShopOrderID());
            // 设置订单商品子订单号
            for (CmOrderProductPo orderProduct : orderProductList) {
                if (shopOrder.getShopID().longValue() == orderProduct.getShopID() && shopOrder.getSplitCode().equals(orderProduct.getSplitCode())) {
                    orderProduct.setShopOrderID(shopOrder.getShopOrderID());
                    orderProduct.setShopOrderNo(shopOrder.getShopOrderNo());
                }
            }
        }

        /**
         * 保存订单商品
         */
        List<OrderProductLadderPricePo> orderProductLadderPriceList = new ArrayList<>();
        for (CmOrderProductPo orderProduct : orderProductList) {
            // 保存订单商品数据
            orderSubmitMapper.insertOrderProduct(orderProduct);
            log.info(">>>>>>>>>>>>>>>>>>>>>>>>>>保存订单商品(insert[cm_order_product])OrderProductID：" + orderProduct.getOrderProductID());
            if ("1".equals(orderProduct.getLadderPriceFlag())) {
                //使用阶梯价格的订单商品保存下单时的阶梯价格列表
                Integer activityId = productMapper.getActivityIdByProductId(orderProduct.getProductID());
                if (activityId != null && activityId > 0) {
                    List<ActivityLadderVo> ladderPriceList = productMapper.getActivityLadderList(activityId, orderProduct.getSkuId());
                    int ladderNum = 0;
                    for (ActivityLadderVo ladder : ladderPriceList) {
                        ladderNum++;
                        OrderProductLadderPricePo orderProductLadderPrice = new OrderProductLadderPricePo();
                        orderProductLadderPrice.setOrderProductId(orderProduct.getOrderProductID());
                        orderProductLadderPrice.setBuyNum(ladder.getBuyNum());
                        orderProductLadderPrice.setBuyPrice(ladder.getBuyPrice());
                        orderProductLadderPrice.setCreateDate(new Date());
                        orderProductLadderPrice.setLadderNum(ladderNum);
                        orderProductLadderPriceList.add(orderProductLadderPrice);
                    }
                }
            }
        }
        if (!CollectionUtils.isEmpty(orderProductLadderPriceList)) {
            orderProductLadderPriceList.forEach(ladderPrice -> {
                orderSubmitMapper.insertOrderProductLadderPrice(ladderPrice);
            });
        }

        // 更新主订单信息, 子订单ID:1000,1002
        order.setShopOrderIDs(shopOrderIds);
        orderSubmitMapper.updateOrder(order);

        /**
         * 保存 订单用户地址
         */
        // 获取地址信息
        AddressVo address = orderSubmitMapper.findByAddressId(addressId);
        if (null != address) {
            //保存地址信息
            BpOrderUserInfoPo userInfo = new BpOrderUserInfoPo();
            userInfo.setOrderId(order.getOrderID());
            userInfo.setUserId(user.getUserID().longValue());
            userInfo.setName(user.getName() == null ? user.getUserName() : user.getName());
            userInfo.setShouHuoRen(address.getShouHuoRen());
            userInfo.setMobile(address.getMobile());
            userInfo.setPostalCode(address.getPhone());
            userInfo.setPostalCode(address.getPostalCode());
            userInfo.setTownId(address.getTownId());
            userInfo.setProvince(address.getProvinceName());
            userInfo.setCity(address.getCityName());
            userInfo.setTown(address.getTownName());
            userInfo.setAddress(address.getAddress());
            orderSubmitMapper.insertUserInfo(userInfo);
            log.info(">>>>>>>>>>>>>>>>>>>>>>>>>>保存订单用户地址(insert[bp_order_userinfo])orderId：" + order.getOrderID());
        } else {
            //设置手动回滚事务
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ResponseJson.error("订单地址异常", null);
        }

        /*
         * 保存优惠券使用情况
         */
        if (null != receiveCouponId && receiveCouponId > 0) {
            // 修改优惠券使用情况
            log.info("保存优惠券使用情况:" + receiveCouponId + ":" + order.getOrderID());
            orderSubmitMapper.updateReceiveCouponStatus(receiveCouponId, order.getOrderID());
            CouponOrderRecordPo orderRecord = new CouponOrderRecordPo();
            orderRecord.setOrderId(order.getOrderID().intValue());
            orderRecord.setReceiveCouponId(receiveCouponId);
            orderRecord.setCouponType(coupon.getCouponType());
            orderRecord.setCouponAmount(coupon.getCouponAmount().doubleValue());
            orderRecord.setTouchPrice(1 == coupon.getNoThresholdFlag() ? 0 : coupon.getTouchPrice().doubleValue());
            orderRecord.setCreateDate(new Date());
            //保存订单优惠记录
            orderSubmitMapper.insertCouponOrderRecord(orderRecord);
        }

        /**
         * 保存拼团成员记录
         */
        if (collageFlag != null && collageFlag == 1) {
            if (orderProductList.size() > 1) {
                //设置手动回滚事务
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return ResponseJson.error("拼团订单商品数量异常", null);
            }
            // 构建拼团成员
            CmHeheCollageMemberPo collageMember = new CmHeheCollageMemberPo();
            collageMember.setUserId(userId);
            collageMember.setOrderId(order.getOrderID());
            if (collageId == null || collageId == 0) {
                // 拼团发起者，构建拼团记录
                CmHeheCollagePo collage = new CmHeheCollagePo();
                CmOrderProductPo orderProductPo = orderProductList.get(0);
                CmHeheCollageProductPo collageProduct = collageMapper.findCollageProduct(orderProductPo.getSkuId());
                if (null == collageProduct) {
                    //设置手动回滚事务
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return ResponseJson.error("拼团商品异常", null);
                }
                collage.setSkuId(orderProductPo.getSkuId());
                collage.setProductId(collageProduct.getProductId());
                collage.setMemberNum(collageProduct.getMemberNum());
                collage.setPrice(collageProduct.getPrice());
                collage.setStatus(0);
                Date startTime = new Date();
                collage.setStartTime(startTime);
                Calendar calendar = Calendar.getInstance();
                calendar.setTime(startTime);
                calendar.add(Calendar.DAY_OF_MONTH, 1);
                collage.setEndTime(calendar.getTime());
                collageMapper.insertCollage(collage);
                collageId = collage.getId();
                collageMember.setLaunchFlag(1);
            } else {
                // 已拼且已支付人数
                Integer memberNum = collageMapper.findCollageMemberNum(collageId);
                CmHeheCollagePo collage = collageMapper.findCollage(collageId);
                if (collage != null && memberNum < collage.getMemberNum()) {
                    // 参加拼团
                    collageMember.setLaunchFlag(0);
                } else {
                    //设置手动回滚事务
                    TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                    return ResponseJson.error("参与拼团异常", null);
                }
            }
            collageMember.setCollageId(collageId);
            collageMapper.insertCollageMember(collageMember);
        }

        /**
         * 保存分享减免订单抵扣记录
         */
        if (null != reductionUserId) {
            orderSubmitMapper.updateReductionUserId(reductionUserId, order.getOrderID());
        }

        log.info("******************** 提交订单逻辑处理 end *******************");
        String openId = orderMapper.getWechatOpenId(userId);
        Map<String,String> mapNames = orderMapper.getProductName(Math.toIntExact(order.getOrderID()));
        String name = mapNames.get("name");
        if (name.length() > 10) {
            name = org.apache.commons.lang.StringUtils.strip(name.substring(0, 11));
            name += "...";
        }
        //微信模板消息
        Map<String,Object> keyWords = new HashMap<>();
        keyWords.put("character_string2",orderNo);
        keyWords.put("thing3",name+"等"+order.getProductCount()+"件商品");
        keyWords.put("amount5","￥"+order.getPayTotalFee()+"元");
        keyWords.put("time6",curDateStr);
        MessageModel<WechatTemplateMessage> wechatTemplateMessageMessageModel = new MessageModel<WechatTemplateMessage>()
                .code(MessageType.WECHAT_TEMPLATE_MESSAGE)
                .mqInfo(new MqInfo().topic("MessageLine").delay(3).async(1))
                .info(new WechatTemplateMessage()
                        .type("2")
                        .templateType("2")
                        .first("下单成功通知")
                        .openId(openId)
                        .appid("wx2c3b0a7f343235b1")
                        .templateId("rIks7f-1-SbTYIPnCQfbfQNhf-hriUk5PJ439TIrDHo")
                        .mapKeyWords(keyWords)
                        .remark("您的订单已成功提交，请尽快完成支付。")
                        .pagePath("pages/views/order/order-detail?orderId="+order.getOrderID()));
        MessageUtil.sendMessage(wechatTemplateMessageMessageModel);
        //站内信
        MessageModel<InsideMessage> insideMessageMessageModel = new MessageModel<>();
        insideMessageMessageModel.code(MessageType.WEB_INSIDE_MESSAGE)
                .mqInfo(new MqInfo().topic("MessageLine").delay(3).async(1))
                .info(new InsideMessage()
                        .userType(4)
                        .messageType(1)
                        .orderMessageType(1)
                        .content("您已成功下单！")
                        .orderId(Math.toIntExact(order.getOrderID()))
                        .thisId(Math.toIntExact(order.getUserID()))
                );
        MessageUtil.sendMessage(insideMessageMessageModel);
        /*
         * 构造返回参数
         */
        Map<String, String> info = new HashMap<>(5);
        info.put("orderId", String.valueOf(order.getOrderID()));
        info.put("orderNo", String.valueOf(order.getOrderNo()));
        info.put("orderMark", "#" + order.getOrderID() + "#");
        //应付订单金额
        info.put("payTotalFee", String.valueOf(order.getPayTotalFee()));
        //真实需要付款金额
        info.put("payableAmount", String.valueOf(order.getPayableAmount()));
        //拼团id
        info.put("collageId", String.valueOf(collageId));
        return ResponseJson.success(info);
    }

    @Override
    public ResponseJson<Integer> shareReduction(Integer userId, Integer reductionId, Integer shareType) {
        // 可参与的分享减免活动
        ReductionVo currentReduction = orderSubmitMapper.findCurrentReduction();
        if (null != currentReduction) {
            // 查询分享次数
            Integer shareNum = orderSubmitMapper.findShareNum(userId, currentReduction.getReductionId());
            if (shareNum <= currentReduction.getShareNum()) {
                ReductionUserPo reductionUserPo = new ReductionUserPo();
                reductionUserPo.setUserId(userId);
                reductionUserPo.setReductionId(reductionId);
                reductionUserPo.setReducedAmount(currentReduction.getReducedAmount());
                reductionUserPo.setTouchPrice(currentReduction.getTouchPrice());
                reductionUserPo.setReductionId(reductionId);
                reductionUserPo.setShareType(shareType);
                orderSubmitMapper.insertReductionUser(reductionUserPo);
                return ResponseJson.success("分享成功", reductionUserPo.getReductionUserId());
            } else {
                return ResponseJson.error("分享次数超限", null);
            }
        } else {
            return ResponseJson.error("分享减免活动不存在", null);
        }
    }

    /**
     * 整理订单商品数据
     *
     * @param productNum
     * @param product
     * @param priceType    0机构价,1活动阶梯价
     * @param productPrice
     * @return
     */
    private CmOrderProductPo setOrderProduct(Integer productNum, CmHeHeProductPo product, int priceType, BigDecimal productPrice, BigDecimal addedValueTax) {
        CmOrderProductPo orderProduct = new CmOrderProductPo();
        orderProduct.setSkuId(product.getSkuId());
        orderProduct.setShopID(product.getShopId().longValue());
        orderProduct.setProductID(product.getProductId());
        orderProduct.setOrganizeProductID(product.getId());
        orderProduct.setSplitCode(product.getSplitCode());
        // 预留在保存保存子订单的时候添加
        orderProduct.setProductNo(null);
        orderProduct.setNum(productNum);
        orderProduct.setPresentNum(0);
        orderProduct.setProductUnit(product.getUnit());
        if (MathUtil.compare(product.getNormalPrice(), 0) > 0) {
            orderProduct.setNormalPrice(product.getNormalPrice());
        } else {
            orderProduct.setNormalPrice(BigDecimal.ZERO);
        }
        orderProduct.setCostPrice(product.getCostPrice());
        orderProduct.setTotalAmount(MathUtil.mul(product.getPrice(), productNum));

        orderProduct.setDiscount(new BigDecimal(100));
        // 经理折扣(优惠金额)
        orderProduct.setDiscountFee(new BigDecimal(0));
        //机构税率
        orderProduct.setTaxRate(product.getClubTaxPoint() == null ? new BigDecimal(0) : product.getClubTaxPoint());
        //供应商税率
        orderProduct.setSupplierTaxRate(BigDecimal.valueOf(0));
        orderProduct.setIncludedTax(product.getIncludedTax().toString());
        orderProduct.setInvoiceType(product.getInvoiceType().toString());
        BigDecimal singleShouldPayTotalTax = new BigDecimal(0);

        //不含税可开发票商品设置税费
        if (0 == product.getIncludedTax() && (1 == product.getInvoiceType() || 2 == product.getInvoiceType())) {
            //供应商税费(单)=成本价 * 供应商税率
            if (product.getShopTaxPoint() == null) {
                orderProduct.setSupplierTaxRate(product.getClubTaxPoint());
            } else {
                orderProduct.setSupplierTaxRate(product.getShopTaxPoint());
            }
            singleShouldPayTotalTax = MathUtil.div(MathUtil.mul(product.getCostPrice(), orderProduct.getSupplierTaxRate()), BigDecimal.valueOf(100));
        } else if (1 == product.getIncludedTax()) {
            //含税商品设置供应商税率
            if (product.getShopTaxPoint() == null) {
                orderProduct.setSupplierTaxRate(product.getClubTaxPoint());
            } else {
                orderProduct.setSupplierTaxRate(product.getShopTaxPoint());
            }
        } else if ((0 == product.getIncludedTax() && 3 == product.getInvoiceType()) || 2 == product.getIncludedTax()) {
            //不含税不可开票商品和未知商品，税率置为0
            orderProduct.setTaxRate(BigDecimal.ZERO);
            orderProduct.setSupplierTaxRate(BigDecimal.ZERO);
        }

        orderProduct.setAddedValueTax(addedValueTax);
        //机构税费(总)=机构税费(单) * 商品数量
        orderProduct.setTotalAddedValueTax(MathUtil.mul(addedValueTax, productNum));
        orderProduct.setSingleShouldPayTotalTax(singleShouldPayTotalTax);
        //供应商税费(总)=供应商税费(单) * 商品数量
        orderProduct.setShouldPayTotalTax(MathUtil.mul(singleShouldPayTotalTax, productNum));
        orderProduct.setTotalFee(MathUtil.mul(productPrice, productNum));
        orderProduct.setShouldPayFee(MathUtil.mul(productPrice, productNum));

        // 商品费=成本价快照*(购买数量  + 赠品数量)
        orderProduct.setShopProductAmount(MathUtil.mul(product.getCostPrice(), BigDecimal.valueOf(productNum)));
        //不含税可开票商品,单价/折后单价=售价-税费

        //阶梯价
        BigDecimal price1 = productPrice;
        //不含税可开票商品，单价/折后单价=售价-税费
        if (0 == product.getIncludedTax() && (1 == product.getInvoiceType() || 2 == product.getInvoiceType())) {
            price1 = MathUtil.sub(productPrice, orderProduct.getAddedValueTax());
        }
        orderProduct.setPrice(price1);
        orderProduct.setDiscountPrice(price1);

        //应付供应商(单)=成本价+供应商税费(单)
        BigDecimal singleShopFee = MathUtil.add(product.getCostPrice(), singleShouldPayTotalTax);
        orderProduct.setSingleShopFee(singleShopFee);
        // 应付供应商(总)=应付供应商(单) * 商品数量
        orderProduct.setShopFee(MathUtil.mul(singleShopFee, BigDecimal.valueOf(productNum)));
        orderProduct.setOtherFee(new BigDecimal(0));
        orderProduct.setSingleOtherFee(new BigDecimal(0));

        //应付采美(单)=单价+机构税费(单)-(成本(单)+供应商税费(单))
        BigDecimal singleCmFee = MathUtil.sub(MathUtil.add(product.getPrice(), orderProduct.getAddedValueTax()), MathUtil.add(product.getCostPrice(), singleShouldPayTotalTax));
        orderProduct.setSingleCmFee(singleCmFee);
        // 应付采美(总)=应付采美(单)*商品数量
        BigDecimal cmFee = MathUtil.mul(singleCmFee, BigDecimal.valueOf(productNum));
        orderProduct.setCmFee(cmFee);

        orderProduct.setTotalBeans(new BigDecimal(0));
        orderProduct.setUseBalanceAmount(0d);
        // 优惠金额
        orderProduct.setPreferential(new BigDecimal(0));
        orderProduct.setUseBalanceAmount(0d);
        // 订单商品供应商确认标志 0否 1是
        orderProduct.setConfirmProductFlag("0");

        // 佣金比例快照
        orderProduct.setCmPercent(product.getCmPercent());
        orderProduct.setOrganizePercent(product.getOrganizePercent());
        orderProduct.setShopPercent(product.getShopPercent());

        orderProduct.setShopName(product.getShopName());
        orderProduct.setName(product.getName());
        orderProduct.setPayStatus("0");
        orderProduct.setBuyAgainFlag("0");
        // 未出库数量
        orderProduct.setNotOutStore(productNum);
        orderProduct.setActPreferential(new BigDecimal(0));
        orderProduct.setActType(null);
        orderProduct.setIsActProduct(priceType + "");
        orderProduct.setLadderPriceFlag("0");
        if (priceType == 1) {
            orderProduct.setLadderPriceFlag("1");
        }
        orderProduct.setProductImage(ProductUtils.getImageURL("product", product.getMainImage(), 0, domain));
        orderProduct.setProductType("0");
        return orderProduct;
    }

    /**
     * 保存子订单，并返回子订单ids
     *
     * @param order
     * @param orderProductList
     * @param shopId
     * @param splitCode
     * @param shopNote
     * @return
     */
    private CmShopOrderPo initShopOrder(CmOrderPo order, List<CmOrderProductPo> orderProductList, Integer shopId, String splitCode, String shopNote) {
        /*
         *  初始化子订单信息
         */
        CmShopOrderPo shopOrder = new CmShopOrderPo();
        // 子订单编号
        String shopOrderNo = "";
        String maxShopOrderNo = orderSubmitMapper.findMaxShopOrderNo(order.getOrderID());
        if (StringUtils.isNotBlank(maxShopOrderNo)) {
            shopOrderNo = maxShopOrderNo;
            shopOrder.setShopOrderNo(OrderNoUtils.getShopOrderNo(order.getOrderNo(), Integer.parseInt(shopOrderNo.substring(shopOrderNo.length() - 2, shopOrderNo.length())) + 1));
        } else {
            shopOrder.setShopOrderNo(OrderNoUtils.getShopOrderNo(order.getOrderNo(), 1));
        }
        shopOrder.setShopID(shopId);
        shopOrder.setOrderID(order.getOrderID());
        shopOrder.setOrderNo(order.getOrderNo());
        shopOrder.setUserID(order.getUserID().intValue());
        shopOrder.setOrganizeID(order.getOrganizeID());
        shopOrder.setSplitCode(splitCode);
        shopOrder.setReceiptAmount(0d);
        shopOrder.setReceiptStatus(1);
        shopOrder.setConfirmTime(new Date());
        /*
         *  统计子订单金额信息
         */
        //运费
        BigDecimal shopPostFee = BigDecimal.ZERO;
        // 订单总金额 + 运费
        BigDecimal totalAmount = MathUtil.add(new BigDecimal(0), shopPostFee);
        // 商品总金额
        BigDecimal productAmount = new BigDecimal(0);
        // 需要支付金额 + 运费
        BigDecimal needPayAmount = MathUtil.add(new BigDecimal(0), shopPostFee);
        // 优惠金额
        BigDecimal preferential = new BigDecimal(0);
        // 佣金
        BigDecimal brokerage = new BigDecimal(0);
        // 商品费
        BigDecimal shopProductAmount = new BigDecimal(0);
        // 供应商税费
        BigDecimal shopTaxFee = new BigDecimal(0);
        // 总购买数
        Integer buyNum = 0;
        // 计算子订单信息
        for (CmOrderProductPo orderProduct : orderProductList) {
            if (shopId.longValue() == orderProduct.getShopID() && splitCode.equals(orderProduct.getSplitCode())) {
                // 商品总金额
                productAmount = MathUtil.add(productAmount, orderProduct.getTotalAmount());
                // 订单总金额 包括税费
                totalAmount = MathUtil.add(totalAmount, orderProduct.getTotalFee());
                // 应付金额
                needPayAmount = MathUtil.add(needPayAmount, orderProduct.getShouldPayFee());
                // 总购买数
                buyNum += orderProduct.getNum();
                preferential = MathUtil.add(preferential, orderProduct.getPreferential());
                brokerage = MathUtil.add(brokerage, orderProduct.getCmFee());
                if (null != orderProduct.getShopProductAmount()) {
                    shopProductAmount = MathUtil.add(shopProductAmount, orderProduct.getShopProductAmount());
                }
                if (null != orderProduct.getShouldPayTotalTax()) {
                    shopTaxFee = MathUtil.add(shopTaxFee, orderProduct.getShouldPayTotalTax());
                }
            }
        }
        //是否冷链费
        shopOrder.setIsColdChina(0);
        shopOrder.setShopPostFlag("0");
        shopOrder.setShopStatus("1");
        shopOrder.setOnlinePayWays("1");



        // freePostFlag: 0包邮 -1到付 1 有运费
        shopOrder.setNote(shopNote);
        shopOrder.setOrderTime(order.getOrderTime());
        shopOrder.setCanRefundFlag(1);
        shopOrder.setCanRefundAmount(needPayAmount.doubleValue());
        shopOrder.setAccountAmount(BigDecimal.ZERO);
        // 佣金 采美应收
        shopOrder.setBrokerage(brokerage);
        shopOrder.setBuyStatus("1");
        shopOrder.setPresentNum(0);
        shopOrder.setUseBeanFlag(0);
        shopOrder.setUseBeanAmount(0);
        shopOrder.setUseBalanceFlag(0);
        shopOrder.setRefundStatus(1);
        shopOrder.setRefundsAmount(BigDecimal.ZERO);
        shopOrder.setPayStatus("1");
        shopOrder.setZeroCostFlag(0);
        shopOrder.setSendOutStatus("1");
        shopOrder.setPayFlag("0");
        // 订单状态标识，1:非退货退款订单、2：退货退款中、3退货退款完成
        shopOrder.setOrderStatusFlag("1");
        shopOrder.setDelFlag("0");
        shopOrder.setOrderBeanAmount(0);
        // 购买商品数
        shopOrder.setItemCount(buyNum);
        // 普通订单 1 协销订单0 与cm_order一样
        shopOrder.setOrderType(order.getOrderType());
        shopOrder.setOrderSubmitType(order.getOrderSubmitType());
        shopOrder.setTotalAmount(totalAmount);
        shopOrder.setProductAmount(productAmount);
        shopOrder.setNeedPayAmount(needPayAmount);
        shopOrder.setPreferential(preferential);
        shopOrder.setShopProductAmount(shopProductAmount);
        // 付给供应商运费
        shopOrder.setShopPostFee(shopPostFee);
        // 付给供应商税费
        shopOrder.setShopTaxFee(shopTaxFee);
        // 已付款金额
        shopOrder.setPayedShopAmount(BigDecimal.ZERO);
        // 付第三方
        shopOrder.setShopOtherFee(BigDecimal.ZERO);
        // 付供应商 = 商品费 + 运费 + 税费
        shopOrder.setShouldPayShopAmount(MathUtil.add(MathUtil.add(shopProductAmount, shopTaxFee), shopPostFee));
        // 订单能否拆分 1 为可拆分, 0为不可拆分
        if (buyNum > 1) {
            shopOrder.setSplitFlag("1");
        } else {
            shopOrder.setSplitFlag("0");
        }
        return shopOrder;
    }
}
