package com.caimei.service.impl;

import com.caimei.mapper.CouponMapper;
import com.caimei.mapper.OrderMapper;
import com.caimei.mapper.ProductMapper;
import com.caimei.mapper.ShoppingCartMapper;
import com.caimei.model.ResponseJson;
import com.caimei.model.dto.CartDto;
import com.caimei.model.po.CmCartPo;
import com.caimei.model.po.CmHeheCollageProductPo;
import com.caimei.model.vo.*;
import com.caimei.service.ShoppingCartService;
import com.caimei.util.MathUtil;
import com.caimei.util.ProductUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.*;

/**
 * Description
 *
 * @author : plf
 * @date : 2021/3/23
 */
@Service
public class ShoppingCartServiceImpl implements ShoppingCartService {
    @Resource
    private ShoppingCartMapper shoppingCartMapper;
    @Resource
    private ProductMapper productMapper;
    @Resource
    private OrderMapper orderMapper;
    @Resource
    private CouponMapper couponMapper;
    @Value("${caimei.oldapi}")
    private String domain;

    @Override
    public ResponseJson<Integer> addShoppingCart(CartDto cart) {
        Integer heUserId = cart.getHeUserId();
        heUserId = heUserId == null ? 0 : heUserId;
        CmCartPo cartPo = shoppingCartMapper.findCartProduct(cart.getUserId(), cart.getSkuId(), heUserId);
        if (cartPo == null) {
            shoppingCartMapper.insertCart(cart);
        } else {
            int productCount = cartPo.getProductCount() + cart.getProductCount();
            shoppingCartMapper.updateByProductCount(cartPo.getCartId(), productCount);
        }
        Integer cartQuantity = shoppingCartMapper.getCartQuantity(cart.getUserId());
        return ResponseJson.success(cartQuantity);
    }

    @Override
    public ResponseJson<Integer> getCartQuantity(Integer userId) {
        Integer cartQuantity = shoppingCartMapper.getCartQuantity(userId);
        return ResponseJson.success(cartQuantity);
    }

    @Override
    public ResponseJson<Map<String, Object>> shoppingInfo(Integer userId) {
        Map<String, Object> map = new HashMap<>(3);
        List<ShopVo> shopList = shoppingCartMapper.findCartShop(userId);
        shopList.forEach(shop -> {
            shop.setLogo(ProductUtils.getImageURL("shopLogo", shop.getLogo(), 0, domain));
            BigDecimal shopTotalPrice = BigDecimal.ZERO;
            List<CartProductVo> productList = shoppingCartMapper.getCartProductsByShopId(shop.getShopId(), userId);
            for (CartProductVo product : productList) {
                setPrice(product, userId, null);
                shopTotalPrice = MathUtil.add(shopTotalPrice, MathUtil.mul(product.getNum(), product.getPrice()));
            }
            shop.setShopTotalPrice(shopTotalPrice);
            shop.setProductList(productList);
        });
        map.put("shopList", shopList);
        //失效商品
        List<CartProductVo> products = shoppingCartMapper.getInvalidProducts(userId);
        map.put("products", products);
        //商品数量
        Integer cartQuantity = shoppingCartMapper.getCartQuantity(userId);
        map.put("cartQuantity", cartQuantity);
        return ResponseJson.success(map);
    }

    /**
     * 设置活动价格阶梯
     */
    @Override
    public void setPrice(CartProductVo product, Integer userId, Integer collageFlag) {
        // 活动id
        Integer activityId = productMapper.getActivityIdByProductId(product.getProductId());
        // 拼团商品
        CmHeheCollageProductPo collageProduct = productMapper.findCollageProduct(product.getSkuId());
        // 限时特价
        BigDecimal discountPrice = productMapper.getDiscountPrice(product.getSkuId());
        // 内部优惠折扣
        HeHeDiscountVo discount = productMapper.findProductDiscount(product.getProductId(), userId);
        if (activityId != null && activityId > 0) {
            product.setActiveStatus(1);
            //活动阶梯
            List<ActivityLadderVo> ladderList = productMapper.getActivityLadderList(activityId, product.getSkuId());
            if (ladderList != null && ladderList.size() > 0) {
                for (ActivityLadderVo ladder : ladderList) {
                    if (product.getNum() >= ladder.getBuyNum()) {
                        // 根据商品数量设置价格
                        product.setPrice(ladder.getBuyPrice());
                    }
                }
            }
            product.setLadderList(ladderList);
        } else if (null != collageProduct && null != collageFlag && 1 == collageFlag) {
            product.setCollageStatus(1);
            product.setPrice(collageProduct.getPrice());
            product.setLimitedNum(1 == collageProduct.getUnlimitedFlag() ? 1000 : collageProduct.getLimitedNum());
        } else if (discountPrice != null) {
            // 限时特价
            product.setPrice(discountPrice);
            product.setDiscountStatus(1);
        } else if (discount != null) {
            BigDecimal price = product.getPrice();
            // 折扣价
            BigDecimal disPrice = discount.getDiscountPrice();
            // 折扣率
            Integer dis = discount.getDiscount();
            if (null != disPrice && disPrice.compareTo(BigDecimal.ZERO)>0) {
                price = disPrice;
            } else if (null != dis && dis > 0) {
                price = MathUtil.div(MathUtil.mul(product.getPrice(), dis), 100);
            }
            product.setPrice(price);
        }
        List<ActivityLadderVo> ladderList = product.getLadderList();
        //税费
        boolean addTaxFlag = ("0".equals(product.getIncludedTax()) && ("1".equals(product.getInvoiceType()) || "2".equals(product.getInvoiceType())));
        // 为商品价格和阶梯列表添加税费
        if (addTaxFlag) {
            BigDecimal addedValueTax = MathUtil.div(MathUtil.mul(product.getPrice(), product.getClubTaxPoint()), BigDecimal.valueOf(100), 2);
            BigDecimal price = MathUtil.add(product.getPrice(), addedValueTax);
            product.setPrice(price);
            addedValueTax = MathUtil.div(MathUtil.mul(product.getNormalPrice(), product.getClubTaxPoint()), BigDecimal.valueOf(100), 2);
            price = MathUtil.add(product.getNormalPrice(), addedValueTax);
            product.setNormalPrice(price);
            if (null != ladderList) {
                ladderList.forEach(ladder->{
                    BigDecimal ladderTax = MathUtil.div(MathUtil.mul(ladder.getBuyPrice(), product.getClubTaxPoint()), BigDecimal.valueOf(100), 2);
                    ladder.setBuyPrice(MathUtil.add(ladder.getBuyPrice(), ladderTax));
                });
            }
        }
        // 查询用户的注册时间
        Date registerTime = couponMapper.getUserRegisterTime(userId);
        String[] productIdArr = {product.getProductId().toString()};
        // 用户在该商品上可领取的优惠券列表
        List<CouponVo> couponList = couponMapper.findCouponList(userId, productIdArr, registerTime, null);
        List<CouponVo> receiveCouponList;
        if (null != userId) {
            // 用户在该商品上已领取未使用的优惠券列表
            receiveCouponList = couponMapper.findReceiveCouponList(userId, productIdArr, null, 1);
            couponList.addAll(receiveCouponList);
        }
        // 单价满足优惠条件的优惠券列表
        List<CouponVo> ableCouponList = new ArrayList<>();
        // 单价不满足优惠条件的优惠券列表
        List<CouponVo> unableCouponList = new ArrayList<>();
        if (couponList.size() > 0) {
            // 有可用优惠券
            couponList.forEach(coupon->{
                if (1 == coupon.getNoThresholdFlag() || MathUtil.compare(product.getPrice(), coupon.getTouchPrice()) >= 0) {
                    ableCouponList.add(coupon);
                } else {
                    unableCouponList.add(coupon);
                }
            });
            if (ableCouponList.size() > 0) {
                // 根据优惠金额排序，优惠金额大的排前面
                ableCouponList.sort((o1, o2) -> o2.getCouponAmount().compareTo(o1.getCouponAmount()));
                CouponVo biggestCoupon = ableCouponList.get(0);
                // 现价-最大优惠金额=券后价
                product.setCouponPrice(MathUtil.sub(product.getPrice(), biggestCoupon.getCouponAmount()));
                // 原价-最大优惠金额=原价券后价
                product.setNormalCouponPrice(MathUtil.sub(product.getNormalPrice(), biggestCoupon.getCouponAmount()));
                //这张优惠券未领取才返回优惠券id显示领券购买
                if (null != biggestCoupon.getUseStatus() && 0 == biggestCoupon.getUseStatus()) {
                    product.setCouponId(biggestCoupon.getCouponId());
                }
                // 券后价标签
                product.setCouponStatus(1);
            } else {
                // 根据优惠条件排序，条件小的排前面
                unableCouponList.sort(Comparator.comparing(CouponVo::getTouchPrice));
                // 优惠券信息
                product.setCouponStatus(2);
                CouponVo smallestCoupon = unableCouponList.get(0);
                product.setCouponInfo("券|满" + smallestCoupon.getTouchPrice() + "元减" + smallestCoupon.getCouponAmount());
            }
        }
        if (1 == product.getActiveStatus() || 1 == product.getCollageStatus() || 1 == product.getDiscountStatus()) {
            if (1 != product.getCollageStatus() && product.getPrice().compareTo(product.getNormalPrice()) == 0) {
                // 原价与售价相同，不显示原价
                product.setNormalPrice(BigDecimal.ZERO);
            }
        } else if (product.getCouponPrice().compareTo(BigDecimal.ZERO) == 0) {
            // 不参与任何活动且未使用优惠券，不显示原价
            product.setNormalPrice(BigDecimal.ZERO);
        }
    }

    @Override
    public ResponseJson<String> buyAgainAddCart(Integer orderId) {
        OrderVo order = orderMapper.findOrder(orderId);
        List<OrderProductVo> orderProductList = orderMapper.findOrderProductAll(orderId);
        CartDto cart = new CartDto();
        cart.setUserId(order.getUserId());
        //商品是否全部下架
        orderProductList.forEach(orderProduct -> {
            Integer validFlag = orderMapper.findProductStatus(orderProduct.getProductId());
            if (validFlag == 1) {
                cart.setProductId(orderProduct.getProductId());
                cart.setProductCount(orderProduct.getNum());
                cart.setSkuId(orderProduct.getSkuId());
                Integer heUserId = orderProduct.getHeUserId();
                if (heUserId != null && heUserId > 0) {
                    Integer activityId = productMapper.getActivityIdByProductId(orderProduct.getProductId());
                    if (activityId != null && activityId > 0) {
                        cart.setHeUserId(heUserId);
                    } else {
                        cart.setHeUserId(0);
                    }
                } else {
                    cart.setHeUserId(0);
                }
                addShoppingCart(cart);
            }
        });
        return ResponseJson.success(null);
    }

    @Override
    public ResponseJson checkOldCart(Integer cartId, Integer newSkuId, Integer count) {
        shoppingCartMapper.updateCart(cartId,newSkuId,count);
        return ResponseJson.success();
    }

    @Override
    public void updateNumber(Integer cartId, Integer productCount) {
        shoppingCartMapper.updateByProductCount(cartId, productCount);
    }

    @Override
    public ResponseJson<String> deleteCart(String cartIds) {
        String[] split = cartIds.split(",");
        for (String cartId : split) {
            if (!StringUtils.isEmpty(cartId)) {
                shoppingCartMapper.deleteCart(cartId);
            }
        }
        return ResponseJson.success("删除购物车成功");
    }
}
